import React, { useState, useCallback, useRef, useEffect } from 'react';
import { View, StyleSheet, Animated, Pressable } from 'react-native';
import { TabBarProps, TabView } from 'react-native-tab-view';

const TABBAR_HEIGHT = 60;

export interface TabViewProps {
    initTabindex?: number;
    tabRoutes: { key: string; title: string }[];
    HeaderLayout?: () => JSX.Element;
    RenderScene: ({
        headerHeight,
        tabBarHeight,
        scrollY,
        onMomentumScrollBegin,
        onMomentumScrollEnd,
        onScrollEndDrag,
        listArrRef,
        tabRoute,
        Key,
        isFocused,
    }: {
        headerHeight: number;
        tabBarHeight: number;
        scrollY: any;
        onMomentumScrollBegin: any;
        onMomentumScrollEnd: any;
        onScrollEndDrag: any;
        listArrRef: any;
        tabRoute: { key: string; title: string };
        Key: string;
        isFocused: boolean;
    }) => JSX.Element | null;
}

const TabViewLayout = (props: TabViewProps) => {
    const { initTabindex, tabRoutes, HeaderLayout, RenderScene } = props;

    const [headerHeight, setHeaderHeight] = useState<number>(0);
    const [tabIndex, setTabIndex] = useState<number>(0);
    const tabIndexRef = useRef<number>(0);
    const isListGlidingRef = useRef<boolean>(false);
    const listArrRef = useRef<any>([]);
    const listOffsetRef = useRef<any>({});

    const scrollY = useRef<any>(new Animated.Value(0)).current;
    const headerTranslateY = scrollY.interpolate({
        inputRange: [0, headerHeight],
        outputRange: [0, -headerHeight],
        extrapolate: 'clamp',
    });

    const tabBarTranslateY = scrollY.interpolate({
        inputRange: [0, headerHeight],
        outputRange: [headerHeight, 0],
        extrapolateRight: 'clamp',
    });

    useEffect(() => {
        if (initTabindex) {
            setTabIndex(initTabindex);
        }
    }, [initTabindex]);

    useEffect(() => {
        scrollY.addListener(({ value }: { value: AsyncGenerator }) => {});

        return () => {
            scrollY.removeListener();
        };
    }, []);

    const headerOnLayout = useCallback((event: any) => {
        const { height } = event.nativeEvent.layout;
        setHeaderHeight(height);
    }, []);

    const onTabIndexChange = useCallback((id: number) => {
        setTabIndex(id);
        tabIndexRef.current = id;
    }, []);

    const onTabPress = useCallback((idx: number) => {
        if (!isListGlidingRef.current) {
            setTabIndex(idx);
            tabIndexRef.current = idx;
        }
    }, []);

    const syncScrollOffset = () => {
        const focusedTabKey = tabRoutes[tabIndexRef.current].key;

        listArrRef.current.forEach((item: any) => {
            if (item.key !== focusedTabKey) {
                if (scrollY._value < headerHeight && scrollY._value >= 0) {
                    if (item.value) {
                        item.value.scrollToOffset({
                            offset: scrollY._value,
                            animated: false,
                        });
                        listOffsetRef.current[item.key] = scrollY._value;
                    }
                } else if (scrollY._value >= headerHeight) {
                    if (listOffsetRef.current[item.key] < headerHeight || listOffsetRef.current[item.key] === null) {
                        if (item.value) {
                            item.value.scrollToOffset({
                                offset: headerHeight,
                                animated: false,
                            });
                            listOffsetRef.current[item.key] = headerHeight;
                        }
                    }
                }
            } else {
                if (item.value) {
                    listOffsetRef.current[item.key] = scrollY._value;
                }
            }
        });
    };

    const onMomentumScrollBegin = useCallback(() => {
        isListGlidingRef.current = true;
    }, []);
    const onMomentumScrollEnd = useCallback(() => {
        isListGlidingRef.current = false;
        syncScrollOffset();
    }, [headerHeight]);
    const onScrollEndDrag = useCallback(() => {
        syncScrollOffset();
    }, [headerHeight]);

    const renderTabBar = (prop: TabBarProps<any>) => {
        const inputRange = prop.navigationState.routes.map((x, i) => i);

        return (
            <Animated.View
                style={[
                    styles.collapsibleTabBar,
                    {
                        transform: [{ translateY: tabBarTranslateY }],
                        elevation: 1,
                        shadowOpacity: 0.5,
                        shadowRadius: 5,
                        shadowOffset: {
                            height: -1,
                            width: 0,
                        },
                        marginBottom: 0,
                    },
                ]}
            >
                {prop.navigationState.routes.map((route, i) => {
                    const opacity = prop.position.interpolate({
                        inputRange,
                        outputRange: inputRange.map((inputIndex) => (inputIndex === i ? 1 : 0.5)),
                    });
                    const color = tabIndex === i ? 'black.200' : 'black.300';
                    const borderColor = tabIndex === i ? 'danzi.100' : 'white';
                    return (
                        <Pressable
                            style={{ flex: 1 }}
                            onPress={() => {
                                //console.log(i);
                                onTabPress(i);
                            }}
                        >
                            <Animated.View
                                style={{
                                    alignItems: 'center',
                                    justifyContent: 'center',
                                    borderColor: borderColor,
                                    opacity: opacity,
                                    flex: 1,
                                    borderBottomWidth: 3,
                                    height: TABBAR_HEIGHT,
                                }}
                            >
                                <Animated.Text
                                    style={{
                                        fontSize: 14,
                                        color: color,
                                    }}
                                >
                                    {route.title}
                                </Animated.Text>
                            </Animated.View>
                        </Pressable>
                    );
                })}
            </Animated.View>
        );
    };

    const renderScene = useCallback(
        ({ route }: any) => {
            const isFocused = route.key === tabRoutes[tabIndex].key;
            return (
                <RenderScene
                    key={route.key}
                    Key={route.key}
                    headerHeight={headerHeight}
                    tabBarHeight={TABBAR_HEIGHT}
                    scrollY={scrollY}
                    onMomentumScrollBegin={onMomentumScrollBegin}
                    onMomentumScrollEnd={onMomentumScrollEnd}
                    onScrollEndDrag={onScrollEndDrag}
                    listArrRef={listArrRef}
                    tabRoute={tabRoutes[tabIndex]}
                    isFocused={isFocused}
                />
            );
        },
        [headerHeight, tabIndex]
    );

    return (
        <View style={styles.rootContainer}>
            {headerHeight > 0 ? (
                <TabView
                    navigationState={{ index: tabIndex, routes: tabRoutes }}
                    renderScene={renderScene}
                    renderTabBar={renderTabBar}
                    onIndexChange={onTabIndexChange}
                />
            ) : null}
            <Animated.View
                style={{ ...styles.headerContainer, transform: [{ translateY: headerTranslateY }] }}
                onLayout={headerOnLayout}
                pointerEvents="box-none"
            >
                {HeaderLayout && <HeaderLayout />}
            </Animated.View>
        </View>
    );
};

export default TabViewLayout;

const styles = StyleSheet.create({
    rootContainer: {
        flex: 1,
    },
    headerContainer: {
        position: 'absolute',
        width: '100%',
    },
    collapsibleTabBar: {
        flexDirection: 'row',
        alignItems: 'center',
        height: TABBAR_HEIGHT,
        backgroundColor: '#FFFFFF',
        zIndex: 1,
    },
    collapsibleTabBarButton: {
        flex: 1,
    },
    collapsibleTabBarLabelContainer: {
        justifyContent: 'center',
        alignItems: 'center',
        height: '100%',
    },
    collapsibleTabBarLabelText: {
        fontSize: 15,
        color: '#587058',
    },
});
